@@ -44,7 +44,7 @@ mystyle_free_back_icon( MyStyle *style )
             style->back_icon.alpha = None ;
         }
 
-        free_icon_resources (style->back_icon);
+        free_icon_resources (&(style->back_icon));
     }
     memset (&(style->back_icon), 0x00, sizeof (style->back_icon));
 }
@@ -442,26 +442,66 @@ clip_root_pixmap( Pixmap root_pixmap, int width, int height )
 	return result;
 }	 
 
-
-ASImage      *
-mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int height, int flip )
+static ASImage      *
+mystyle_make_image_int (MyStyle * style, int root_x, int root_y, int crop_x, int crop_y, int crop_width, int crop_height, int scale_width, int scale_height, int flip, ASImage *underlayment, int overlay_type )
 {
 	ASImage      *im = NULL;
 	Pixmap        root_pixmap;
-	Bool transparency = False ;
+	Bool transparency = TransparentMS(style) ;
 	int preflip_width, preflip_height ;
+	int width, height ; 
+	
+	if(underlayment ) 
+	{
+		if( overlay_type >= TEXTURE_SCALED_TRANSPIXMAP )
+			overlay_type = TEXTURE_TRANSPIXMAP + (overlay_type - TEXTURE_SCALED_TRANSPIXMAP);
+		if( overlay_type < TEXTURE_TRANSPIXMAP || overlay_type >= TEXTURE_TRANSPIXMAP_END ) 
+			overlay_type = TEXTURE_TRANSPIXMAP_ALPHA ;
+		
+		if( transparency )
+			transparency = (overlay_type != TEXTURE_TRANSPIXMAP_ALPHA);
+	} 
 
+	if( crop_x < 0 ) 
+	{
+		root_x += crop_x ; 
+		width = scale_width-crop_x ; 
+		crop_x = 0 ;
+		if( width < crop_width )
+			width = crop_width;
+	}else
+	{
+		width = crop_width + crop_x ; 
+		if( width < scale_width ) 
+			width = scale_width ;
+	}
+
+	if( crop_y < 0 ) 
+	{
+		root_y += crop_y ; 
+		height = scale_height-crop_y ; 
+		crop_y = 0 ;
+		if( height < crop_height )
+			height = crop_height;
+	}else
+	{
+		height = crop_height + crop_y ; 
+		if( height < scale_height ) 
+			height = scale_height ;
+	}
+	
 	if (width < 1)
 		width = 1;
 	if (height < 1)
 		height = 1;
+		
+		
     LOCAL_DEBUG_OUT ("style \"%s\", texture_type = %d, im = %p, tint = 0x%lX, geom=(%dx%d%+d%+d), flip = %d", style->name, style->texture_type,
                      style->back_icon.image, style->tint, width, height, root_x, root_y, flip);
 
-	transparency = TransparentMS(style);
-
-	if(  transparency )
+	if( transparency )
 	{	/* in this case we need valid root image : */
+		
 		if (ASDefaultScr->RootImage == NULL)
 		{
 			unsigned int  root_w, root_h;
@@ -553,13 +593,43 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 	{
 		if (ASDefaultScr->RootImage != NULL)
 		{
-			if (style->texture_type == TEXTURE_TRANSPARENT || style->texture_type == TEXTURE_TRANSPARENT_TWOWAY)
+			ASImage *root_im = ASDefaultScr->RootImage;
+			int root_src_x = root_x - ASDefaultScr->RootClipArea.x;
+			int root_src_y = root_y - ASDefaultScr->RootClipArea.y;
+			Bool do_tint = (style->texture_type == TEXTURE_TRANSPARENT || style->texture_type == TEXTURE_TRANSPARENT_TWOWAY);
+			Bool do_blur = (get_flags (style->set_flags, F_BLUR) && (style->blur_x > 1 || style->blur_y > 1));
+			LOCAL_DEBUG_OUT( "root+im = %p, blur is %s set, size = %dx%d", root_im, get_flags (style->set_flags, F_BLUR)?"is":"not", style->blur_x, style->blur_y);
+			if ( do_tint 
+			     || (do_blur && (root_src_x != 0 || root_src_y != 0 
+			                     || width != root_im->width || height != root_im->height)))
 			{
-                im = tile_asimage ( ASDefaultVisual, ASDefaultScr->RootImage, 
-									root_x-ASDefaultScr->RootClipArea.x, root_y-ASDefaultScr->RootClipArea.y,
-									width, height, style->tint, 
-									ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
-			} else
+                ASImage *tmp = tile_asimage ( ASDefaultVisual, root_im,
+											  root_src_x, root_src_y,
+									          width, height, do_tint?style->tint:TINT_LEAVE_SAME, 
+									          ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+				LOCAL_DEBUG_OUT( "tint result = %p", tmp );
+				if (tmp)
+				{
+					root_src_x = root_src_y = 0;
+					root_im = tmp;
+				}
+			}
+			if (do_blur)
+			{
+#if 1
+				ASImage *tmp = blur_asimage_gauss( ASDefaultVisual, root_im, style->blur_x, style->blur_y, 0xFFFFFFFF, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+				LOCAL_DEBUG_OUT( "blur result = %p", tmp );
+				if (tmp && tmp != root_im)
+				{
+					if (root_im != ASDefaultScr->RootImage)
+						destroy_asimage (&root_im);
+					root_im = tmp;
+				}
+#endif
+			}
+			if (do_tint)
+				im = root_im;
+			else
 			{
 				 ASImageLayer  layers[2];
 				 ASImage      *scaled_im = NULL;
@@ -568,7 +638,7 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 
 				 init_image_layers (&layers[0], 2);
 
-				 layers[0].im = ASDefaultScr->RootImage;
+				 layers[0].im = root_im;
 				 if (style->texture_type == TEXTURE_SHAPED_SCALED_PIXMAP &&
 					 style->texture_type == TEXTURE_SHAPED_PIXMAP)
 				 {
@@ -583,8 +653,8 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 				 layers[0].merge_scanlines = mystyle_merge_scanlines_func_xref[index];
                  layers[0].dst_x = 0;
                  layers[0].dst_y = 0;
-                 layers[0].clip_x = root_x-ASDefaultScr->RootClipArea.x;
-                 layers[0].clip_y = root_y-ASDefaultScr->RootClipArea.y;
+                 layers[0].clip_x = root_src_x;
+                 layers[0].clip_y = root_src_y;
 				 layers[0].clip_width = width;
 				 layers[0].clip_height = height;
 
@@ -600,16 +670,21 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 												style->slice_y_start, style->slice_y_end,
 												preflip_width, preflip_height, do_scale, 
 												ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);			   
+ 					LOCAL_DEBUG_OUT( "image scliced to %p", scaled_im ) ;
 				}else if( do_scale )  
 				{
 					scaled_im = scale_asimage ( ASDefaultVisual, layers[1].im, 
 												preflip_width, preflip_height,
 											    ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
- 				}
+ 					LOCAL_DEBUG_OUT( "image scaled to %p", scaled_im ) ;
+				}
+				
 				 if (scaled_im)	 
 				 {	
 					 /* here layers[1].im is always style->back_icon.image, so we should not destroy it ! */
-					 layers[1].im = mystyle_flip_image( scaled_im, width, height, flip );
+					 scaled_im = mystyle_flip_image( scaled_im, width, height, flip );
+					 layers[1].im = scaled_im ;
+					 LOCAL_DEBUG_OUT( "image flipped to %p", layers[1].im ) ;
 					 /* scaled_im got destroyed in mystyle_flip_image if it had to be */
 				 }else if( flip != 0 && layers[1].im == style->back_icon.image )
 				 {
@@ -642,7 +717,8 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 								LOCAL_DEBUG_OUT( "copying alpha channel from %p to %p", im, tmp );
 								copy_asimage_channel(tmp, IC_ALPHA, im, IC_ALPHA);
 							}
-							safe_asimage_destroy (im);
+							 if ( im != ASDefaultScr->RootImage)
+								safe_asimage_destroy (im);
 						}
 						im = tmp ;
 					}
@@ -650,6 +726,8 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 				if (scaled_im)
 					destroy_asimage (&scaled_im);
 			 }
+			 if ( root_im && root_im != ASDefaultScr->RootImage && root_im != im)
+			 	destroy_asimage( &root_im);
 		}else
 			show_warning( "MyStyle \"%s\" : failed to accure Root background image", style->name );
 	}
@@ -660,9 +738,105 @@ mystyle_make_image (MyStyle * style, int root_x, int root_y, int width, int heig
 		im->back_color = style->colors.back ;
 		show_warning( "MyStyle \"%s\" : failed to generate background image - using solid fill instead with color #0x%8.8X", style->name, style->colors.back );
 	}
+
+	if( underlayment )
+	{
+		 ASImageLayer  layers[2];
+		 int           index = 1;      /* default is alphablending !!! */
+
+		 init_image_layers (&layers[0], 2);
+
+		 layers[0].im = underlayment;
+		 index = overlay_type - TEXTURE_TRANSPIXMAP;
+
+		 LOCAL_DEBUG_OUT ("index = %d", index);
+		 layers[0].merge_scanlines = mystyle_merge_scanlines_func_xref[index];
+         layers[0].dst_x = 0;
+         layers[0].dst_y = 0;
+         layers[0].clip_x = 0;
+         layers[0].clip_y = 0;
+		 layers[0].clip_width = width;
+		 layers[0].clip_height = height;
+
+		 layers[1].im = im;
+		 layers[1].merge_scanlines = layers[0].merge_scanlines;
+		 layers[1].dst_x = 0;
+		 layers[1].dst_y = 0;
+		 layers[1].clip_x = 0;
+		 layers[1].clip_y = 0;
+		 layers[1].clip_width = width;
+		 layers[1].clip_height = height;
+
+		{
+			ASImage *tmp = merge_layers(ASDefaultVisual, &layers[0], 2, 
+										width, height, 
+										ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+			if( tmp )
+			{
+				if( im )
+				{
+					if (style->texture_type == TEXTURE_SHAPED_SCALED_PIXMAP ||
+						style->texture_type == TEXTURE_SHAPED_PIXMAP)
+					{
+						/*we need to keep alpha channel intact */
+						LOCAL_DEBUG_OUT( "copying alpha channel from %p to %p", im, tmp );
+						copy_asimage_channel(tmp, IC_ALPHA, im, IC_ALPHA);
+					}
+					if ( im != ASDefaultScr->RootImage)
+						safe_asimage_destroy (im);
+				}
+				im = tmp ;
+			}
+		}
+	}
+
+	if( style->overlay ) 
+	{
+		ASImage *overlayed = mystyle_make_image_int (style->overlay, root_x, root_y, crop_x, crop_y, crop_width, crop_height, scale_width, scale_height, flip, im, style->overlay_type );
+/* 		fprintf( stderr, "overlay_style = %p\n", style->overlay ); */
+		if( overlayed ) 
+		{
+			if( im && overlayed != im && im != ASDefaultScr->RootImage) 
+				safe_asimage_destroy (im);
+			im = overlayed ;
+		}			
+	}
+	if( crop_x > 0 || crop_y > 0 || 
+		(crop_width > 0 && crop_width < im->width ) ||
+		(crop_height > 0 && crop_height < im->height ) ) 
+	{
+		ASImage *cropped_im = tile_asimage( ASDefaultVisual, im,
+											crop_x, crop_y,
+											crop_width, crop_height,
+											TINT_LEAVE_SAME,
+											ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+		if( cropped_im ) 
+		{
+			if( im && cropped_im != im && im != ASDefaultScr->RootImage ) 
+				safe_asimage_destroy (im);
+			im = cropped_im ;
+		}			
+	}
 	return im;
 }
 
+ASImage      *
+mystyle_make_image(MyStyle * style, int root_x, int root_y, int width, int height, int flip )
+{
+	if( style != NULL ) 
+		return mystyle_make_image_int (style, root_x, root_y, 0, 0, width, height, width, height, flip, NULL, 0 );
+	return NULL ;
+}
+
+ASImage      *
+mystyle_crop_image(MyStyle * style, int root_x, int root_y, int crop_x, int crop_y, int width, int height, int scale_width, int scale_height, int flip )
+{
+	if( style != NULL ) 
+		return mystyle_make_image_int (style, root_x, root_y, crop_x, crop_y, width, height, scale_width, scale_height, flip, NULL, 0 );
+	return NULL ;
+}
+
+
 /*************************************************************************/
 /* Mystyle creation/deletion                                             */
 /*************************************************************************/
@@ -671,7 +845,7 @@ mystyle_free_resources( MyStyle *style )
 {
     if( style->magic == MAGIC_MYSTYLE )
     {
-        if (style->user_flags & F_FONT)
+        if (get_flags( style->user_flags, F_FONT))
         {
             unload_font (&style->font);
         }
@@ -863,33 +1037,30 @@ mystyle_find_or_default (const char *name)
  *   inherit if the parent style could go away before the child
  */
 void
-mystyle_merge_font( MyStyle *style, MyFont *font, Bool override, Bool copy)
+mystyle_merge_font( MyStyle *style, MyFont *font, Bool override)
 {
-    if ((override == True) && (style->user_flags & F_FONT))
+    if ( override && get_flags(style->user_flags, F_FONT))
+	{
         unload_font (&style->font);
-    if ((override == True) || !(style->set_flags & F_FONT))
+        clear_flags(style->user_flags, F_FONT);
+        clear_flags(style->set_flags, F_FONT);
+	}
+    if ( override || !get_flags(style->set_flags, F_FONT) )
     {
-        if (copy == False)
-        {
-            style->font = *font;
-            style->user_flags &= ~F_FONT;
-            style->inherit_flags |= F_FONT;
-        } else
-        {
-            if (load_font (font->name, &style->font) == True)
-            {
-                style->user_flags |= F_FONT;
-                style->inherit_flags &= ~F_FONT;
-            }
-        }
+		set_string( &(style->font.name), mystrdup(font->name));
+		style->font.as_font = dup_asfont( font->as_font );
+        set_flags(style->user_flags, F_FONT);
+        clear_flags(style->inherit_flags, F_FONT);
     }
 }
 
 void
 mystyle_merge_styles (MyStyle * parent, MyStyle * child, Bool override, Bool copy)
 {
+	if( parent == NULL || child == NULL ) 
+		return;
 	if (parent->set_flags & F_FONT)
-        mystyle_merge_font( child, &(parent->font), override, copy);
+        mystyle_merge_font( child, &(parent->font), override);
 
     if (parent->set_flags & F_TEXTSTYLE)
 	{
@@ -1063,6 +1234,25 @@ mystyle_merge_styles (MyStyle * parent, MyStyle * child, Bool override, Bool cop
 			}
 		}
 	}
+	if (parent->set_flags & F_OVERLAY)
+	{
+		if ((override == True) || !(child->set_flags & F_OVERLAY))
+		{
+			child->flags &= ~F_OVERLAY;
+			child->flags |= parent->flags & F_OVERLAY;
+			child->overlay = parent->overlay ;
+			child->overlay_type = parent->overlay_type ;
+			if (copy == False)
+			{
+				child->user_flags &= ~F_OVERLAY;
+				child->inherit_flags |= F_OVERLAY;
+			} else
+			{
+				child->user_flags |= F_OVERLAY;
+				child->inherit_flags &= ~F_OVERLAY;
+			}
+		}
+	}
 	child->set_flags = child->user_flags | child->inherit_flags;
 }
 
@@ -1119,6 +1309,8 @@ void
 mystyle_merge_colors (MyStyle * style, int type, char *fore, char *back,
 					   char *gradient, char *pixmap)
 {
+	if( style == NULL ) 
+		return ;
 	if ((fore != NULL) && !((*style).user_flags & F_FORECOLOR))
 	{
 		if (parse_argb_color (fore, &((*style).colors.fore)) != fore)
@@ -1204,9 +1396,10 @@ mystyle_inherit_font (MyStyle * style, MyFont * font)
 	 *       unloaded once */
 	if (style != NULL && !(style->set_flags & F_FONT))
 	{
-		style->font = *font;
-		style->inherit_flags |= F_FONT;
-		style->user_flags &= ~F_FONT;		   /* to prevent confusion */
+		set_string( &(style->font.name), font->name );
+		style->font.as_font = dup_asfont(font->as_font);
+		clear_flags(style->inherit_flags, F_FONT);
+		set_flags(style->user_flags, F_FONT);		   /* to prevent confusion */
 		style->set_flags = style->user_flags | style->inherit_flags;
 	}
 }
@@ -1328,6 +1521,10 @@ mystyle_draw_text_image (MyStyle * style, const char *text, unsigned long encodi
 
 	if (style && text)
 	{
+		/* load fonts on demand only - no need to waste memory if ain't never gonna use it ! */
+		if ( style->font.as_font == NULL )
+			load_font( NULL, &style->font );
+
 		if (style->font.as_font)
         {
 			ASTextAttributes attr = {ASTA_VERSION_1, ASTA_UseTabStops, AST_Plain, ASCT_Char, 8, 0, NULL, 0, ARGB32_White };
@@ -1358,7 +1555,26 @@ unsigned int
 mystyle_get_font_height (MyStyle * style)
 {
 	if (style)
+	{
+		if (style->font.as_font == NULL )
+			load_font( NULL, &style->font );
+
 		if (style->font.as_font)
 			return style->font.as_font->max_height;
+	}
 	return 1;
 }
+
+void
+mystyle_get_text_size (MyStyle * style, const char *text, unsigned int *width, unsigned int *height )
+{
+	if (style && text)
+	{
+		if (style->font.as_font == NULL )
+			load_font( NULL, &style->font );
+
+		if (style->font.as_font)
+			get_text_size( text, style->font.as_font, style->text_style, width, height );
+	}
+}
+
